# Sara Sprenkle for Fall 2007 class

"""Module containing classes that represent various aspects of
music."""

from counter import *

LIBRARY_DIR="libraries/"
ALBUM_DIR="albums/"

class Song:

    """ Class representing a song."""
    
    def __init__(self, title, artist, album, minutes, seconds):
        """Create a new song object.  title, artist, and album are all strings.
        minutes and seconds are integers."""
        self.artist = artist
        self.album = album
        self.title = title
        self.playtime = PlayTime(0, 0, minutes, seconds)

    def __str__(self):
        rep = "Song: \"" + self.title + "\""
        rep += "\n\toff the album \"" + self.album + "\""
        rep += "\n\tby " + self.artist
        return rep

    def getPlaytime(self):
        return self.playtime

    def __cmp__(self, other):
        "Sorts songs by song title, album name, then artist name"
        othertitle = other.getTitle()
        if self.title < othertitle:
            return -1
        if self.title > othertitle:
            return 1
        
        otherartist = other.getArtistName()
        if self.artist < otherartist:
            return -1
        if self.artist > otherartist:
            return 1
            
        otheralbum = other.getAlbumName()
        if self.album < otheralbum:
            return -1
        if self.album > otheralbum:
            return 1
            
        return 0

    def getAlbumName(self):
        return self.album

    def getArtistName(self):
        return self.artist

    def getTitle(self):
        return self.title

    def writeToFile(self, outfile):
        """Writes a song to the given file object."""
        outfile.write(self.artist + "\n")
        outfile.write(self.album + "\n")
        outfile.write(self.title + "\n")
        outfile.write("%02d:%02d\n"% (self.playtime.getMinutes(), self.playtime.getSeconds()))
        

class MusicLibrary:
    """Represents a music library."""
    
    def __init__(self, filename):
        self.songs = []

        if not filename.startswith(LIBRARY_DIR):
            filename = LIBRARY_DIR + filename
        
        self.filename = filename
        self.playtime = PlayTime()
        self._readLibrary()

    def __str__(self):
        rep = ""
        for song in self.songs:
            rep += str(song) + "\n"
        rep += str(self.playtime) + " total time\n"
        return rep
        
    def prettyPrint(self):
        print "%-35s %-20s %-20s %-8s" %("Title", "Artist", "Album", "Length")
        print "-"*88
        for song in self.songs:
            print "%-35s %-20s %-20s %-8s" %(song.getTitle(), song.getArtistName(), song.getAlbumName(), song.getPlaytime())
        print "-"*88
        print "Total: ", len(self.songs), "songs",
        print "Playtime: ", self.playtime

    def _readLibrary(self):
        library = file(self.filename,"r")

        numsongsstr = library.readline().strip()
        numsongs = int(numsongsstr)
        print "\tLoading library from", self.filename
        # read in each song from the library
        for x in xrange(numsongs):
            song = self._readInSongFromLibrary(library)
            self.songs.append(song)
            self.playtime.addPlayTime(song.getPlaytime())
        
        library.close()

    def _readInSongFromLibrary(self, infile):
        """Creates (and returns) a song from reading in the song's
        information from the infile file object."""
        
        artist = infile.readline().strip()
        album = infile.readline().strip()
        title = infile.readline().strip()
        lengthstr = infile.readline().strip()
        min = int(lengthstr[:lengthstr.find(":")])
        sec = int(lengthstr[lengthstr.find(":")+1:])
        
        song = Song(title, artist, album, min, sec)
        return song
    
    def sort(self):
        self.songs.sort()
        
    def storeLibrary(self, filename=None):
        """Write the library to a file.  If no filename given, 
        writes to its library file."""

        # handle default: write to library file
        if filename is None:
            filename = self.filename
        # otherwise, check if filename is in appropriate directory
        elif not filename.startswith(LIBRARY_DIR):
            filename = LIBRARY_DIR + filename

        library = file(filename, "w")
            
        print "\tSaving library to file:", filename

        # write the number of songs to the export file
        library.write(str(len(self.songs)) + "\n")

        # write each song to the export file
        for song in self.songs:
            song.writeToFile(library)
            
        library.close()

    def importAlbum(self, filename):
        """Imports the songs from an album file."""

        if not filename.startswith(ALBUM_DIR):
            filename = ALBUM_DIR + filename
        
        album = file(filename,"r")

        print 
        print "\tReading album from file", filename
        artist = album.readline().strip()
        albumname = album.readline().strip()
        numsongsstr = album.readline().strip()
        numsongs = int(numsongsstr)

        albumPT = PlayTime()
        
        for x in xrange(numsongs):
            title = album.readline().strip()
            lengthstr = album.readline().strip()
            min = int(lengthstr[:lengthstr.find(":")])
            sec = int(lengthstr[lengthstr.find(":")+1:])
            
            song = Song(title, artist, albumname, min, sec)
            self.songs.append(song)
            albumPT.addPlayTime(song.getPlaytime())
        album.close()
        print "\tSuccessfully imported", numsongs, "songs,",
        print "totalling", albumPT

        self.playtime.addPlayTime(albumPT)
        

class PlayTime:
    # by Sara Sprenkle

    """A class that represents the playing time of music in days,
    hours, minutes, and seconds."""
    
    def __init__(self, days=0, hours=0, minutes=0, seconds=0):
        """Construct a PlayTime object.  days, hours, minutes, and
        seconds are integers.  By default, they're all 0. """
        self.seconds = Counter(0,59)
        self.seconds.setValue(seconds)
        self.minutes = Counter(0,59)
        self.minutes.setValue(minutes)
        self.hours = Counter(0,23)
        self.hours.setValue(hours)
        self.days = 0

    def __str__(self):
        rep = ""
        if self.days > 1:
            rep += "%d days " % self.days
        elif self.days == 1:
            rep += "1 day "
        hours = self.hours.getValue()
        mins = self.minutes.getValue()
        secs = self.seconds.getValue()
        rep += ("%02d:%02d:%02d" % (hours, mins, secs))
        return rep

    def getMinutes(self):
        "Returns the number of minutes in the PlayTime"
        return self.minutes.getValue()

    def getSeconds(self):
        "Returns the number of seconds in the PlayTime"
        return self.seconds.getValue()

    def getHours(self):
        "Returns the number of hours in the PlayTime"
        return self.hours.getValue()

    def getDays(self):
        "Returns the number of days in the PlayTime"
        return self.days

    def addPlayTime(self, playtime):
        """Adds the PlayTime object playtime to this PlayTime object."""
        return self.add(playtime.getDays(), playtime.getHours(), playtime.getMinutes(), playtime.getSeconds())

    def add(self, days, hours, minutes, seconds):
        "Adds the days, hours, minutes and seconds to this Playtime object"
        # increment each part
        numWraps1 = self.seconds.increment(seconds)
        numWraps2 = self.minutes.increment(minutes)
        numWraps3 = self.hours.increment(hours)
        self.days += days
        # handle the "carries"
        if numWraps1 > 0:
            numWraps2 += self.minutes.increment(numWraps1)
        if numWraps2 > 0:
            numWraps3 += self.hours.increment(numWraps2)
        if numWraps3 > 0:
            self.days += numWraps3

def testPlayTime():
    pt = PlayTime()
    print pt
    
    pt.add(0,0,2,30)
    print pt
    
    #test minute wrap
    pt.add(0,0,2,30)
    print pt
    
    pt.addPlayTime(pt)
    print pt
    
    # test hour wrap
    print "Test hour wrap.."
    pt.add(0,0,50,0)
    print pt
    # test minute wrap
    pt.addPlayTime(pt)
    print pt
    
    # test day wrap
    pt.add(0, 22, 0, 1)
    print pt
    pt.addPlayTime(pt)
    print pt
    
    pt2 = PlayTime(0, 23, 59, 59)
    print pt2
    print "Testing rollover..."
    pt2.add(0,0,0,1)
    print pt2


def testMusic():
    artist1= "Bon Jovi"
    album1 = "Bon Jovi"
    song1 = Song("You Give Love a Bad Name", artist1, album1, 3, 22)
    print song1
    song2 = Song("Blaze of Glory", artist1, album1, 3, 10)
    print song2
    song3 = Song("Spiderwebs", "No Doubt", "Tragic Kingdom", 3, 30)
    album = Album(artist1, album1)
    album.addSong(song1)
    album.addSong(song2)
    album.addSong(song3)
    print album

    print song1 < song2
    print song2 < song3
    print song3 > song1

def testCollection():
    mytunes = MusicLibrary()
    mytunes.readLibrary("mytunes.library")
    print mytunes
    mytunes.storeLibrary("mytunes2.library")
    mytunes.addAlbum("data/killers.txt")
    mytunes.addAlbum("data/west.txt")
    print mytunes
    mytunes.storeLibrary()


#testMusic()
#testCollection()
#testPlayTime()
